/*
 * Copyright (C) 2002-2005 Stefan Stiller
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.kiwisoft.sqlPlugin.actions;

import java.awt.event.ActionEvent;
import java.net.URL;
import javax.help.CSH;
import javax.help.HelpBroker;
import javax.help.HelpSet;
import javax.help.HelpSetException;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;

import com.kiwisoft.sqlPlugin.Icons;
import com.kiwisoft.utils.gui.IconManager;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.3 $, $Date: 2006/02/18 15:31:52 $
 */
public class HelpAction extends AnAction
{
	private static CSH.DisplayHelpFromSource displayHelpFromSource;

	public HelpAction()
	{
		super("Help");
		getTemplatePresentation().setIcon(IconManager.getIcon(Icons.HELP));
		getTemplatePresentation().setDescription("Help about using the plugin.");
		if (displayHelpFromSource==null)
		{
			try
			{
				URL hsURL=HelpSet.findHelpSet(HelpAction.class.getClassLoader(), "SQLPlugin/help.hs");
				HelpSet helpSet=new HelpSet(null, hsURL);
				HelpBroker helpBroker=helpSet.createHelpBroker();
				displayHelpFromSource=new CSH.DisplayHelpFromSource(helpBroker);
			}
			catch (HelpSetException e)
			{
				e.printStackTrace();
				getTemplatePresentation().setEnabled(false);
			}
		}
	}

	public void actionPerformed(AnActionEvent event)
	{
		if (displayHelpFromSource!=null)
		{
			ActionEvent actionEvent=new ActionEvent(event.getInputEvent().getSource(), ActionEvent.ACTION_PERFORMED, "Help");
			displayHelpFromSource.actionPerformed(actionEvent);
		}
	}
}
