/*
 * Copyright (C) 2002-2003 Stefan Stiller
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.kiwisoft.sqlPlugin.dataLoad;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.io.BufferedReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Vector;
import javax.swing.*;
import javax.swing.table.AbstractTableModel;

import com.intellij.openapi.vfs.VirtualFile;

import com.kiwisoft.wizard.WizardDialog;
import com.kiwisoft.wizard.WizardPane;
import com.kiwisoft.sqlPlugin.dataLoad.io.CSVReader;
import com.kiwisoft.utils.gui.table.TableUtils;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.3 $, $Date: 2002/02/22 10:00:24 $
 */
public class DataSourcePage extends WizardPane
{
	private JComboBox formatField;

	private JPanel fileTypePanel;
	private CSVComponent csvComponent;
	private FixedWidthComponent fixedWidthComponent;
	private VirtualFile file;

	public DataSourcePage(WizardDialog wizardDialog, VirtualFile file)
	{
		super(wizardDialog);
		this.file=file;
	}

	public String getTitle()
	{
		return "Setup Data Source";
	}

	public JComponent createComponent()
	{
		List lines=new ArrayList();
		try
		{
			BufferedReader reader=new BufferedReader(file.getReader());
			String line;
			while (lines.size()<50 && (line=reader.readLine())!=null) lines.add(line);
			reader.close();
		}
		catch (IOException e)
		{
			throw new RuntimeException(e);
		}

		String[] lineArray=(String[])lines.toArray(new String[0]);
		csvComponent=new CSVComponent();
		fixedWidthComponent=new FixedWidthComponent(lineArray);

		fileTypePanel=new JPanel(new GridBagLayout());
		formatField=new JComboBox(new Object[]{csvComponent, fixedWidthComponent});
		formatField.setEditable(false);
		FormatListener formatListener=new FormatListener();
		getListenerSupport().installActionListener(formatField, formatListener);
		formatListener.actionPerformed(null);


		JPanel panel=new JPanel(new GridBagLayout());
		int row=0;
		panel.add(new JLabel("File Format:"), new GridBagConstraints(0, row, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
		panel.add(formatField, new GridBagConstraints(1, row, 1, 1, 1.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 0, 5, 5), 0, 0));
		row++;
		panel.add(fileTypePanel, new GridBagConstraints(0, row, 2, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));
		return panel;
	}

	protected boolean canGoForward()
	{
		return true;
	}

	private static abstract class FileTypeComponent
	{
		protected abstract void showComponent(JPanel fileTypePanel) throws IOException;
	}

	private class CSVComponent extends FileTypeComponent implements FocusListener, ActionListener
	{
		private JComboBox delimiterField;
		private JComboBox qualifierField;
		private JTextField rowsToSkipField;
		private JTable table;
		private JCheckBox titleRowField;

		public CSVComponent()
		{
			delimiterField=new JComboBox(new Object[]{
				new CharacterResource('\t', "Tabulator"),
				new CharacterResource(',', "Comma"),
				new CharacterResource(';', "Semicolon"),
				new CharacterResource(' ', "Space"),
				new CharacterResource('|', "Pipe")
			});
			delimiterField.addActionListener(this);
			qualifierField=new JComboBox(new Object[]{
				new CharacterResource('"', "Quote"),
				new CharacterResource('\'', "Single Quotes"),
				new CharacterResource((char)0, "None")
			});
			qualifierField.addActionListener(this);
			rowsToSkipField=new JTextField(3);
			rowsToSkipField.setHorizontalAlignment(SwingConstants.RIGHT);
			rowsToSkipField.setMinimumSize(new Dimension(30, rowsToSkipField.getPreferredSize().height));
			rowsToSkipField.addFocusListener(this);
			titleRowField=new JCheckBox("Title Row");
			titleRowField.addActionListener(this);
		}

		public String toString()
		{
			return "CSV";
		}

		protected void showComponent(JPanel panel) throws IOException
		{
			table=new JTable();
			table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
			focusLost(null);

			int row=0;
			panel.add(new JLabel("Delimiter:"), new GridBagConstraints(0, row, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
			panel.add(delimiterField, new GridBagConstraints(1, row, 1, 1, 1.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 0, 5, 5), 0, 0));
			panel.add(new JLabel("Qualifier:"), new GridBagConstraints(2, row, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
			panel.add(qualifierField, new GridBagConstraints(3, row, 1, 1, 1.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 0, 5, 5), 0, 0));

			row++;
			panel.add(new JLabel("# of Rows to Skip:"), new GridBagConstraints(0, row, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
			panel.add(rowsToSkipField, new GridBagConstraints(1, row, 1, 1, 1.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 5, 5), 0, 0));
			panel.add(titleRowField, new GridBagConstraints(3, row, 1, 1, 1.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0, 5, 5), 0, 0));

			row++;
			panel.add(new JScrollPane(table), new GridBagConstraints(0, row, 4, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));
		}

		public void actionPerformed(ActionEvent e)
		{
			updateTable();
		}

		public void focusGained(FocusEvent e)
		{
		}

		public void focusLost(FocusEvent e)
		{
			updateTable();
		}

		private void updateTable()
		{
			try
			{
				CharacterResource resource=(CharacterResource)delimiterField.getSelectedItem();
				char delimiter=resource!=null ? resource.getCharacter() : 0;
				resource=(CharacterResource)qualifierField.getSelectedItem();
				char qualifier=resource!=null ? resource.getCharacter() : 0;
				int rowsToSkip;
				try
				{
					rowsToSkip=Integer.parseInt(rowsToSkipField.getText());
				}
				catch (NumberFormatException e)
				{
					rowsToSkip=0;
				}
				boolean titleRow=titleRowField.isSelected();

				CSVReader reader=new CSVReader(new BufferedReader(file.getReader()), delimiter, qualifier);
				String[] row;
				String[] titles=null;
				Vector data=new Vector();
				int columns=0;
				while (data.size()<50 && (row=reader.readRow())!=null)
				{
					if (rowsToSkip>0)
					{
						rowsToSkip--;
						continue;
					}
					if (titleRow)
					{
						titles=row;
						titleRow=false;
						continue;
					}
					if (row.length>columns) columns=row.length;
					data.add(new Vector(Arrays.asList(row)));
				}
				reader.close();
				table.setModel(new CSVPreviewModel(titles, columns, data));
				TableUtils.sizeColumnsToFit(table, true, true);
			}
			catch (Exception e1)
			{
				e1.printStackTrace();
			}
		}

		private class CSVPreviewModel extends AbstractTableModel
		{
			private String[] titles;
			private final int columns;
			private final Vector data;

			public CSVPreviewModel(String[] titles, int columns, Vector data)
			{
				this.titles=titles;
				this.columns=columns;
				this.data=data;
			}

			public String getColumnName(int column)
			{
				if (titles!=null && column<titles.length) return titles[column];
				return "Column #"+column;
			}

			public int getColumnCount()
			{
				return columns;
			}

			public int getRowCount()
			{
				return data.size();
			}

			public Object getValueAt(int rowIndex, int columnIndex)
			{
				if (rowIndex<data.size())
				{
					Vector row=(Vector)data.get(rowIndex);
					if (columnIndex<row.size())
					{
						return row.get(columnIndex);
					}
				}
				return null;
			}
		}
	}

	private class FixedWidthComponent extends FileTypeComponent
	{
		private ColumnInitializer columnInitializer;

		public FixedWidthComponent(String[] lines)
		{
			columnInitializer=new ColumnInitializer(lines);
		}

		public String toString()
		{
			return "Fixed Width";
		}

		protected void showComponent(JPanel panel)
		{
			JScrollPane scrollPane=new JScrollPane(columnInitializer);
			panel.add(scrollPane, new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));
		}
	}

	private class FormatListener implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			FileTypeComponent fileTypeComponent=(FileTypeComponent)formatField.getSelectedItem();
			fileTypePanel.removeAll();
			try
			{
				if (fileTypeComponent!=null) fileTypeComponent.showComponent(fileTypePanel);
			}
			catch (IOException e1)
			{
				e1.printStackTrace();
			}
			fileTypePanel.updateUI();
			validateActions();
		}
	}
}
