package com.kiwisoft.sqlPlugin.dataLoad;

import com.kiwisoft.utils.gui.table.*;
import com.kiwisoft.sqlPlugin.DefaultTableConfiguration;

import javax.swing.*;
import java.awt.*;

/**
 * Created by IntelliJ IDEA.
 * User: Gieselbert
 * Date: 04.03.2006
 * Time: 09:43:34
 * To change this template use File | Settings | File Templates.
 */
public class SourceColumnsTableModel extends SortableTableModel
{
    public static final String[] COLUMNS = {"column", "type", "pattern", "trim"};

    public static void main(String[] args)
    {
        EditorFactory.getInstance().setEditor(String.class, DataType.NUMBER + "Format", new NumberFormatLookup());
        EditorFactory.getInstance().setEditor(String.class, DataType.DATE + "Format", new DateFormatLookup());
        EditorFactory.getInstance().setEditor(String.class, DataType.FLAG + "Format", new BooleanFormatLookup());
        EditorFactory.getInstance().setDefaultEditor(DataType.class,
                new ChoiceObjectEditor(new Object[]{DataType.TEXT, DataType.NUMBER, DataType.DATE, DataType.FLAG},
                        false));

        JFrame frame = new JFrame();
        frame.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        SourceColumnsTableModel model = new SourceColumnsTableModel();
        for (int i = 1; i <= 10; i++)
        {
            SourceColumnDescriptor columnDescriptor = new SourceColumnDescriptor();
            columnDescriptor.setName("Column " + i);
            columnDescriptor.setType(DataType.TEXT);
            model.addRow(new Row(columnDescriptor));
        }
        SortableTable table = new SortableTable(model);
        table.initializeColumns(new DefaultTableConfiguration("sourceColumns"));
        table.sizeColumnsToFit(true, true);
        table.putClientProperty(SortableTable.ALTERNATE_ROW_COLOR, new Color(235, 255, 235));
        frame.setContentPane(new JScrollPane(table));
        frame.pack();
        frame.setVisible(true);
    }

    public int getColumnCount()
    {
        return COLUMNS.length;
    }

    public String getColumnName(int column)
    {
        return COLUMNS[column];
    }

    private static class Row extends SortableTableRow
    {
        public Row(SourceColumnDescriptor columnDescriptor)
        {
            super(columnDescriptor);
        }

        public SourceColumnDescriptor getDescriptor()
        {
            return (SourceColumnDescriptor) getUserObject();
        }

        public String getCellFormat(int col)
        {
            if (col == 2)
            {
                DataType type = getDescriptor().getType();
                if (type == null) return null;
                else return type.toString() + "Format";
            }
            return super.getCellFormat(col);
        }

        public Class getCellClass(int col)
        {
            switch (col)
            {
                case 0:
                case 2:
                    return String.class;
                case 1:
                    return DataType.class;
                case 3:
                    return Boolean.class;
            }
            return super.getCellClass(col);
        }

        public Object getDisplayValue(int column)
        {
            switch (column)
            {
                case 0:
                    return getDescriptor().getName();
                case 1:
                    return getDescriptor().getType();
                case 2:
                    return getDescriptor().getPattern();
                case 3:
                    return Boolean.valueOf(getDescriptor().isTrim());
            }
            return null;
        }

        public int setValue(Object value, int col)
        {
            switch (col)
            {
                case 0:
                    getDescriptor().setName((String) value);
                    return CELL_UPDATE;
                case 1:
                    getDescriptor().setType((DataType) value);
                    return ROW_UPDATE;
                case 2:
                    getDescriptor().setPattern((String) value);
                    return CELL_UPDATE;
                case 3:
                    getDescriptor().setTrim(Boolean.TRUE.equals(value));
                    return CELL_UPDATE;
            }
            return super.setValue(value, col);
        }

        public boolean isEditable(int column)
        {
            if (column == 2)
            {
                DataType type = getDescriptor().getType();
                return type!=null && type.usePattern();
            }
            return true;
        }
    }
}
