/*
 * Copyright (C) 2002-2003 Stefan Stiller
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.kiwisoft.sqlPlugin.dataLoad.io;

import com.kiwisoft.utils.xml.XMLAdapter;
import com.kiwisoft.utils.xml.XMLContext;
import com.kiwisoft.utils.xml.XMLHandler;
import com.kiwisoft.utils.xml.XMLObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.io.File;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.3 $, $Date: 2002/02/22 10:00:24 $
 */
public class FixedWidthFormat extends XMLAdapter
{
    private List elements=new ArrayList();
    private Map recordTypes=new HashMap();

    public FixedWidthFormat(XMLContext dummy, String aName)
    {
        super(dummy, aName);
    }

    public void addXMLElement(XMLContext context, XMLObject element)
    {
        if (element instanceof RecordElement)
            elements.add(element);
        else if (element instanceof RecordType)
        {
            RecordType type=(RecordType) element;
            recordTypes.put(type.getId(), type);
        }
        else super.addXMLElement(context, element);
    }

    public List getElements()
    {
        return elements;
    }

    public RecordType getRecordType(String id)
    {
        RecordType type=(RecordType)recordTypes.get(id);
        if (type==null) type=(RecordType)recordTypes.get("*");
        return type;
    }

    private static XMLHandler createHandler()
    {
        XMLHandler xmlHandler=new XMLHandler();
        xmlHandler.addTagMapping("fileFormat", FixedWidthFormat.class);
        xmlHandler.addTagMapping("element", RecordElement.class);
        xmlHandler.addTagMapping("recordType", RecordType.class);
        return xmlHandler;
    }

    public static FixedWidthFormat load(Class aClass, String resource)
    {
        XMLHandler xmlHandler=createHandler();
        return (FixedWidthFormat) xmlHandler.loadStream(aClass.getResourceAsStream(resource));
    }

    public static FixedWidthFormat load(File file)
    {
        XMLHandler xmlHandler=createHandler();
        return (FixedWidthFormat) xmlHandler.loadFile(file);
    }
}
