/*
 * Copyright (C) 2002-2003 Stefan Stiller
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.kiwisoft.sqlPlugin.dataLoad.io;

import java.io.*;
import java.util.Map;
import java.util.LinkedHashMap;
import java.util.Iterator;
import java.text.ParseException;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.3 $, $Date: 2002/02/22 10:00:24 $
 */
public class FixedWidthReader
{
    private FixedWidthFormat format;
	private BufferedReader reader;

	protected FixedWidthReader(FixedWidthFormat format, File file) throws FileNotFoundException
	{
		if (format==null) throw new IllegalArgumentException("'format' must not be null.");
		if (file==null) throw new IllegalArgumentException("'file' must not be null.");
		this.format=format;
		reader=new BufferedReader(new FileReader(file));
	}

	public Map readLine() throws IOException, ParseException
	{
		String line=reader.readLine();
		if (line==null) return null;
		Map data=new LinkedHashMap();
		int position=0;
		String recordTypeId=null;
		for (Iterator it=format.getElements().iterator(); it.hasNext();)
		{
			RecordElement element=(RecordElement)it.next();
			String stringValue=line.substring(position, position+=element.getLength());
			stringValue=convertToUnicode(stringValue, false);
			if (element.isTrim()) stringValue=stringValue.trim();
			data.put(element.getName(), element.convert(stringValue));
			if (element.isRecordType()) recordTypeId=stringValue;
		}
		RecordType recordType=null;
		if (recordTypeId!=null)
		{
			recordType=format.getRecordType(recordTypeId);
			for (Iterator it=recordType.getElements().iterator(); it.hasNext();)
			{
				RecordElement element=(RecordElement)it.next();
				String stringValue=line.substring(position, position+=element.getLength());
				stringValue=FixedWidthReader.convertToUnicode(stringValue, false);
				if (element.isTrim()) stringValue=stringValue.trim();
				data.put(element.getName(), element.convert(stringValue));
			}
		}
		return data;
	}

    private static final String specialSaveChars = "= \t\r\n\f#!";

    /*
      * Converts unicodes to encoded &#92;uxxxx
      * and writes out any of the characters in specialSaveChars
      * with a preceding slash
      */
    public static String convertToUnicode(String theString, boolean escapeSpace)
    {
        int len=theString.length();
        StringBuffer outBuffer=new StringBuffer(len*2);

        for (int x=0; x<len; x++)
        {
            char aChar=theString.charAt(x);
            switch (aChar)
            {
                case ' ':
                    if (escapeSpace) outBuffer.append('\\');
                    outBuffer.append(' ');
                    break;
                case '\\':
                    outBuffer.append('\\');
                    outBuffer.append('\\');
                    break;
                case '\t':
                    outBuffer.append('\\');
                    outBuffer.append('t');
                    break;
                case '\n':
                    outBuffer.append('\\');
                    outBuffer.append('n');
                    break;
                case '\r':
                    outBuffer.append('\\');
                    outBuffer.append('r');
                    break;
                case '\f':
                    outBuffer.append('\\');
                    outBuffer.append('f');
                    break;
                default:
                    if ((aChar<0x0020) || (aChar>0x007e))
                    {
                        outBuffer.append('\\');
                        outBuffer.append('u');
                        outBuffer.append(Integer.toHexString((aChar>> 12)&0xF));
                        outBuffer.append(Integer.toHexString((aChar>> 8)&0xF));
                        outBuffer.append(Integer.toHexString((aChar>> 4)&0xF));
                        outBuffer.append(Integer.toHexString(aChar&0xF));
                    }
                    else
                    {
                        if (FixedWidthReader.specialSaveChars.indexOf(aChar)!=-1)
                            outBuffer.append('\\');
                        outBuffer.append(aChar);
                    }
            }
        }
        return outBuffer.toString();
    }
}
