/*
 * Copyright (c) 2006 Your Corporation. All Rights Reserved.
 */
package com.kiwisoft.sqlPlugin.dataLoad.io;

import com.kiwisoft.utils.StringUtils;
import com.kiwisoft.utils.BooleanFormat;
import com.kiwisoft.utils.xml.XMLAdapter;
import com.kiwisoft.utils.xml.XMLContext;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.text.DecimalFormat;

/**
 * @author Stefan Stiller
 * @version $Revision: $, $Date: $
 * @since 17.01.2005
 */
public class RecordElement extends XMLAdapter
{
    public static final String DATE="date";
    public static final String INT="int";
    public static final String DECIMAL="decimal";
    public static final String TEXT="text";
    public static final String FLAG="flag";

    private String name;
    private int length;
    private boolean recordType;
    private boolean trim=true;
    private String type=TEXT;
    private String pattern;

    public RecordElement(XMLContext dummy, String aName)
    {
        super(dummy, aName);
    }

    public void setXMLAttribute(XMLContext context, String name, String value)
    {
        if ("name".equalsIgnoreCase(name)) this.name=value;
        else if ("length".equalsIgnoreCase(name)) this.length=Integer.parseInt(value);
        else if ("recordType".equalsIgnoreCase(name)) this.recordType=Boolean.valueOf(value).booleanValue();
        else if ("trim".equalsIgnoreCase(name)) this.trim=Boolean.valueOf(value).booleanValue();
        else if ("type".equalsIgnoreCase(name)) this.type=value;
        else if ("pattern".equalsIgnoreCase(name)) this.pattern=value;
        else super.setXMLAttribute(context, name, value);
    }

    public boolean isTrim()
    {
        return trim;
    }

    public String getName()
    {
        return name;
    }

    public int getLength()
    {
        return length;
    }

    public boolean isRecordType()
    {
        return recordType;
    }

    public Object convert(String stringValue) throws ParseException
    {
        if (TEXT.equalsIgnoreCase(type)) return stringValue;
        else if (DATE.equalsIgnoreCase(type))
        {
            if (pattern==null) throw new RuntimeException("Missing pattern for element: "+getName());
            if (StringUtils.isEmpty(stringValue)) return null;
            return new SimpleDateFormat(pattern).parse(stringValue);
        }
        else if (DECIMAL.equalsIgnoreCase(type))
        {
            if (StringUtils.isEmpty(stringValue)) return null;
            if (pattern==null) return new Double(stringValue);
            else return new DecimalFormat(pattern).parse(stringValue);
        }
        else if (INT.equalsIgnoreCase(type))
        {
            if (StringUtils.isEmpty(stringValue)) return null;
            if (pattern==null) return new Long(stringValue);
            else return new DecimalFormat(pattern).parse(stringValue);
        }
        else if (FLAG.equalsIgnoreCase(type))
        {
            if (pattern==null) return Boolean.valueOf(stringValue);
            else return new BooleanFormat(pattern).parse(stringValue);
        }
        throw new RuntimeException("Ivalid type: "+type);
    }

    public String toString()
    {
        return name;
    }
}
