/*
 * Copyright (C) 2002-2005 Stefan Stiller
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.kiwisoft.sqlPlugin.settings;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.util.*;
import javax.swing.*;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;

import com.kiwisoft.utils.StringUtils;
import com.kiwisoft.utils.SetMap;
import com.kiwisoft.utils.format.ObjectFormat;
import com.kiwisoft.utils.gui.table.RendererFactory;
import com.kiwisoft.sqlPlugin.config.SQLPluginAppConfig;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.6 $, $Date: 2006/02/18 15:31:53 $
 */
public class ResultPropertiesPanel extends JPanel
{
	private JCheckBox cbPrimaryKeys;
	private JCheckBox cbResizeColumnsToHeader;
	private JCheckBox cbResizeColumnsToContent;
	private JCheckBox cbSaveTableConfiguration;
	private JTextField tfNullString;
	private JComboBox cbxDateFormats;
	private JComboBox cbxNumberFormats;
	private JComboBox cbxStringFormats;
	private JComboBox cbxBooleanFormats;
	private JCheckBox cbRowLimit;
	private JLabel lblMaxRows;
	private JTextField tfMaxRows;
	private JComboBox cbxPartialQuery;
	private JCheckBox cbSaveQueries;
	private JCheckBox cbProjectClassLoader;
	private JCheckBox cbLoadLOBs;
	private JCheckBox cbStopOnError;

	private ItemListener rowLimitListener;

	public ResultPropertiesPanel()
	{
		initializeComponents();
		installListeners();
		initializeData();
	}

	private void initializeData()
	{
		SQLPluginAppConfig configuration=SQLPluginAppConfig.getInstance();
		cbRowLimit.setSelected(configuration.isRowLimitEnabled());
		cbSaveQueries.setSelected(configuration.isSaveQueries());
		cbProjectClassLoader.setSelected(configuration.isIncludeProjectClasses());
		cbStopOnError.setSelected(configuration.isStopOnError());
		tfMaxRows.setEnabled(configuration.isRowLimitEnabled());
		lblMaxRows.setEnabled(configuration.isRowLimitEnabled());
		tfMaxRows.setText(Integer.toString(configuration.getRowLimit()));
		selectBoolean(cbxPartialQuery, configuration.getConfirmation(SQLPluginAppConfig.PARTIAL_EXECUTE));
		cbPrimaryKeys.setSelected(configuration.isShowPrimaryKeyColumns());
		cbLoadLOBs.setSelected(configuration.isLoadLargeObjects());
		cbResizeColumnsToContent.setSelected(configuration.isResizeColumnsToContent());
		cbResizeColumnsToHeader.setSelected(configuration.isResizeColumnsToHeader());
		cbSaveTableConfiguration.setSelected(configuration.isSaveResultTableConfiguration());
		tfNullString.setText(configuration.getNullString());
		String format=configuration.getDefaultFormat(Date.class);
		if (format!=null) cbxDateFormats.setSelectedItem(format);
		else cbxDateFormats.setSelectedIndex(0);
		format=configuration.getDefaultFormat(String.class);
		if (format!=null) cbxStringFormats.setSelectedItem(format);
		else cbxStringFormats.setSelectedIndex(0);
		format=configuration.getDefaultFormat(Number.class);
		if (format!=null) cbxNumberFormats.setSelectedItem(format);
		else cbxNumberFormats.setSelectedIndex(0);
		format=configuration.getDefaultFormat(Boolean.class);
		if (format!=null) cbxBooleanFormats.setSelectedItem(format);
		else cbxBooleanFormats.setSelectedIndex(0);
	}

	private void selectBoolean(JComboBox comboBox, Boolean value)
	{
		DefaultComboBoxModel comboBoxModel=(DefaultComboBoxModel)comboBox.getModel();
		NamedBoolean item=new NamedBoolean("", value);
		int index=comboBoxModel.getIndexOf(item);
		if (index<0)
		{
			if (value==null) comboBox.setSelectedIndex(0);
			else selectBoolean(comboBox, null);
		}
		comboBox.setSelectedIndex(index);
	}

	private void initializeComponents()
	{
		lblMaxRows=new JLabel("Number of Rows:");
		tfMaxRows=new JTextField(5);
		tfMaxRows.setHorizontalAlignment(SwingConstants.TRAILING);
		cbSaveQueries=new JCheckBox("Save into workspace");
		cbProjectClassLoader=new JCheckBox("Include project classes for BLOB's");
		cbStopOnError=new JCheckBox("Stop on Error");

		cbxPartialQuery=new JComboBox(new Object[]{
			new NamedBoolean("Ask", null),
			new NamedBoolean("Execute part of query", Boolean.TRUE),
			new NamedBoolean("Execute complete query", Boolean.FALSE)
		});

		cbRowLimit=new JCheckBox("Restrict number of rows loaded");
		cbRowLimit.setHorizontalTextPosition(SwingConstants.TRAILING);
		cbRowLimit.setEnabled(true);

		cbPrimaryKeys=new JCheckBox("Highlight primary key columns");
		cbPrimaryKeys.setHorizontalTextPosition(SwingConstants.TRAILING);

		cbLoadLOBs=new JCheckBox("Load LOB's");
		cbLoadLOBs.setHorizontalTextPosition(SwingConstants.TRAILING);

		cbSaveTableConfiguration=new JCheckBox("Save last table settings into workspace");
		cbSaveTableConfiguration.setHorizontalTextPosition(SwingConstants.TRAILING);

		cbxDateFormats=new JComboBox(getFormats(Date.class));
		cbxNumberFormats=new JComboBox(getFormats(Number.class));
		cbxBooleanFormats=new JComboBox(getFormats(Boolean.class));
		cbxStringFormats=new JComboBox(getFormats(String.class));

		cbResizeColumnsToHeader=new JCheckBox("...to fit table header");
		cbResizeColumnsToContent=new JCheckBox("...to fit table content");

		tfNullString=new JTextField(10);

		JPanel pnlQueries=new JPanel();
		int row=0;
		pnlQueries.setLayout(new GridBagLayout());
		pnlQueries.setBorder(new TitledBorder(new EtchedBorder(), "Queries"));
		pnlQueries.add(cbRowLimit,
					   new GridBagConstraints(0, row++, 2, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(4, 4, 2, 4), 0, 0));
		pnlQueries.add(lblMaxRows,
					   new GridBagConstraints(0, row, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
		pnlQueries.add(tfMaxRows,
					   new GridBagConstraints(1, row++, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
		pnlQueries.add(new JLabel("Partial queries:"),
					   new GridBagConstraints(0, row, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
		pnlQueries.add(cbxPartialQuery,
					   new GridBagConstraints(1, row++, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
		pnlQueries.add(cbSaveQueries,
					   new GridBagConstraints(0, row, 2, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 4, 4, 4), 0, 0));
		pnlQueries.add(cbStopOnError,
					   new GridBagConstraints(2, row++, 2, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 4, 4, 4), 0, 0));
		pnlQueries.add(cbLoadLOBs,
					   new GridBagConstraints(0, row, 2, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 4, 4, 4), 0, 0));
		pnlQueries.add(cbProjectClassLoader,
					   new GridBagConstraints(2, row++, 2, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 4, 4, 4), 0, 0));
		pnlQueries.add(Box.createGlue(),
					   new GridBagConstraints(2, row, 1, 1, 1.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));

		JPanel pnlResult=new JPanel();
		pnlResult.setLayout(new GridBagLayout());
		pnlResult.setBorder(new TitledBorder(new EtchedBorder(), "Result Table"));
		row=0;
		pnlResult.add(cbSaveTableConfiguration,
					  new GridBagConstraints(0, row++, 2, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
		pnlResult.add(cbPrimaryKeys,
					  new GridBagConstraints(0, row++, 2, 1, 0.5, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
		pnlResult.add(new JLabel("Automatically resize columns..."),
					  new GridBagConstraints(0, row++, 2, 1, 1.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
		pnlResult.add(cbResizeColumnsToHeader,
					  new GridBagConstraints(0, row++, 2, 1, 0.5, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 24, 4, 4), 0, 0));
		pnlResult.add(cbResizeColumnsToContent,
					  new GridBagConstraints(0, row++, 2, 1, 0.5, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 24, 4, 4), 0, 0));
		pnlResult.add(new JLabel("Null String:"),
					  new GridBagConstraints(0, row, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
		pnlResult.add(tfNullString,
					  new GridBagConstraints(1, row, 1, 1, 0.5, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));

		JPanel pnlFormats=new JPanel();
		pnlFormats.setLayout(new GridBagLayout());
		pnlFormats.setBorder(new TitledBorder(new EtchedBorder(), "Default Formats"));
		row=0;
		pnlFormats.add(new JLabel("Strings:"),
					   new GridBagConstraints(0, row, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
		pnlFormats.add(cbxStringFormats,
					   new GridBagConstraints(1, row, 1, 1, 1.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
		pnlFormats.add(new JLabel("Numbers:"),
					   new GridBagConstraints(2, row, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
		pnlFormats.add(cbxNumberFormats,
					   new GridBagConstraints(3, row++, 1, 1, 1.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
		pnlFormats.add(new JLabel("Booleans:"),
					   new GridBagConstraints(0, row, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
		pnlFormats.add(cbxBooleanFormats,
					   new GridBagConstraints(1, row, 1, 1, 1.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
		pnlFormats.add(new JLabel("Dates:"),
					   new GridBagConstraints(2, row, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(4, 4, 4, 4), 0, 0));
		pnlFormats.add(cbxDateFormats,
					   new GridBagConstraints(3, row, 1, 1, 1.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));

		setLayout(new GridBagLayout());
		row=0;
		add(pnlQueries,
			new GridBagConstraints(0, row++, 1, 1, 1.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(7, 7, 7, 7), 0, 0));
		add(pnlResult,
			new GridBagConstraints(0, row++, 1, 1, 1.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 7, 7, 7), 0, 0));
		add(pnlFormats,
			new GridBagConstraints(0, row++, 1, 1, 1.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 7, 7, 7), 0, 0));
		add(Box.createGlue(),
			new GridBagConstraints(0, row, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
	}

	private static Vector getFormats(Class aClass)
	{
		SetMap variants=RendererFactory.getInstance().getVariants(aClass);
		Vector list=new Vector();
		for (Iterator it=variants.keySet().iterator(); it.hasNext();)
		{
			list.addAll(variants.get(it.next()));
		}
		list.remove(ObjectFormat.DEFAULT);
		Collections.sort(list);
		list.add(0, ObjectFormat.DEFAULT);
		return list;
	}

	private void installListeners()
	{
		rowLimitListener=new RowLimitListener();
		cbRowLimit.addItemListener(rowLimitListener);
	}

	private void removeListeners()
	{
		cbRowLimit.removeItemListener(rowLimitListener);
	}

	public void removeNotify()
	{
		removeListeners();
	}

	public boolean canApply()
	{
		boolean rowLimit=cbRowLimit.isSelected();
		if (rowLimit)
		{
			try
			{
				Integer.parseInt(tfMaxRows.getText());
			}
			catch (NumberFormatException e)
			{
				JOptionPane.showMessageDialog(this, e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
				tfMaxRows.requestFocus();
				return false;
			}
		}
		return true;
	}

	public void apply()
	{
		SQLPluginAppConfig configuration=SQLPluginAppConfig.getInstance();

		int maxRows=1000;
		boolean rowLimit=cbRowLimit.isSelected();
		Object selectedItem=cbxPartialQuery.getSelectedItem();
		Boolean partialExecute=null;
		if (selectedItem instanceof NamedBoolean) partialExecute=((NamedBoolean)selectedItem).getValue();
		if (rowLimit)
		{
			maxRows=Integer.parseInt(tfMaxRows.getText());
		}

		String nullString=tfNullString.getText();
		if (StringUtils.isEmpty("nullString")) nullString=null;
		configuration.setNullString(nullString);
		configuration.setShowPrimaryKeyColumns(cbPrimaryKeys.isSelected());
		configuration.setDefaultFormat(Date.class, (String)cbxDateFormats.getSelectedItem());
		configuration.setDefaultFormat(Number.class, (String)cbxNumberFormats.getSelectedItem());
		configuration.setDefaultFormat(String.class, (String)cbxStringFormats.getSelectedItem());
		configuration.setDefaultFormat(Boolean.class, (String)cbxBooleanFormats.getSelectedItem());
		configuration.setResizeColumnsToContent(cbResizeColumnsToContent.isSelected());
		configuration.setResizeColumnsToHeader(cbResizeColumnsToHeader.isSelected());
		configuration.setSaveResultTableConfiguration(cbSaveTableConfiguration.isSelected());
		configuration.setSaveQueries(cbSaveQueries.isSelected());
		configuration.setIncludeProjectClasses(cbProjectClassLoader.isSelected());
		configuration.setLoadLargeObjects(cbLoadLOBs.isSelected());
		configuration.setConfirmation(SQLPluginAppConfig.PARTIAL_EXECUTE, partialExecute);
		configuration.setStopOnError(cbStopOnError.isSelected());
		configuration.setRowLimitEnabled(rowLimit);
		if (rowLimit) configuration.setRowLimit(maxRows);
	}

	private class RowLimitListener implements ItemListener
	{
		public void itemStateChanged(ItemEvent e)
		{
			boolean selected=e.getStateChange() == ItemEvent.SELECTED;
			lblMaxRows.setEnabled(selected);
			tfMaxRows.setEnabled(selected);
		}
	}

	private static class NamedBoolean
	{
		private String label;
		private Boolean value;

		public NamedBoolean(String label, Boolean value)
		{
			this.label=label;
			this.value=value;
		}

		public String toString()
		{
			return label;
		}

		public Boolean getValue()
		{
			return value;
		}

		public boolean equals(Object o)
		{
			if (this==o) return true;
			if (!(o instanceof NamedBoolean)) return false;

			final NamedBoolean namedBoolean=(NamedBoolean)o;

			return !(value!=null ? !value.equals(namedBoolean.value) : namedBoolean.value!=null);
		}

		public int hashCode()
		{
			return value!=null ? value.hashCode() : 0;
		}
	}
}
