/*
 * Copyright (C) 2002-2005 Stefan Stiller
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.kiwisoft.utils;

import java.awt.Component;
import java.awt.event.ActionListener;
import java.awt.event.MouseListener;
import java.awt.event.KeyListener;
import java.util.Observable;
import java.util.Observer;
import java.util.Stack;
import javax.swing.JComboBox;
import javax.swing.JTabbedPane;
import javax.swing.text.JTextComponent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentListener;

import com.kiwisoft.utils.gui.lookup.DialogLookupField;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.3 $, $Date: 2005/08/10 18:50:21 $
 */
public class ListenerSupport
{
	private Stack disposables=new Stack();

	public ListenerSupport()
	{
	}

	public void dispose()
	{
		while (!disposables.isEmpty())
		{
			Disposable disposable=(Disposable)disposables.pop();
			disposable.dispose();
		}
	}

	public void installObserver(final Observable observable, final Observer observer)
	{
		observable.addObserver(observer);
		disposables.push(new Disposable()
		{
			public void dispose()
			{
				observable.deleteObserver(observer);
			}
		});
	}

	public void installActionListener(final JComboBox comboBox, final ActionListener actionListener)
	{
		comboBox.addActionListener(actionListener);
		disposables.push(new Disposable()
		{
			public void dispose()
			{
				comboBox.removeActionListener(actionListener);
			}
		});
	}

	public void installChangeListener(final JTabbedPane tabbedPane, final ChangeListener changeListener)
	{
		tabbedPane.addChangeListener(changeListener);
		disposables.push(new Disposable()
		{
			public void dispose()
			{
				tabbedPane.removeChangeListener(changeListener);
			}
		});
	}

	public void installMouseListener(final Component component, final MouseListener mouseListener)
	{
		component.addMouseListener(mouseListener);
		disposables.push(new Disposable()
		{
			public void dispose()
			{
				component.removeMouseListener(mouseListener);
			}
		});
	}

	public void installKeyListener(final Component component, final KeyListener keyListener)
	{
		component.addKeyListener(keyListener);
		disposables.push(new Disposable()
		{
			public void dispose()
			{
				component.removeKeyListener(keyListener);
			}
		});
	}

	public void installDocumentListener(final JTextComponent component, final DocumentListener listener)
	{
		component.getDocument().addDocumentListener(listener);
		disposables.push(new Disposable()
		{
			public void dispose()
			{
				component.getDocument().removeDocumentListener(listener);
			}
		});
	}

	public void installDocumentListener(final DialogLookupField component, final DocumentListener listener)
	{
		component.addDocumentListener(listener);
		disposables.push(new Disposable()
		{
			public void dispose()
			{
				component.removeDocumentListener(listener);
			}
		});
	}
}
