/*
 * Created by IntelliJ IDEA.
 * User: Stefan1
 * Date: Apr 5, 2003
 * Time: 2:25:05 PM
 */
package com.kiwisoft.utils.gui.lookup;

import java.awt.BorderLayout;
import java.awt.Point;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import javax.swing.AbstractAction;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JWindow;
import javax.swing.KeyStroke;

import com.kiwisoft.utils.StringComparator;

public abstract class ListLookup implements Lookup
{
	private LookupField field;
	private JList list;

    protected ListLookup()
    {
    }

	public void open(LookupField field, Collection values)
	{
		this.field=field;
		LookupWindow window = new LookupWindow(values);
		Point point=field.getLocationOnScreen();
		window.setLocation(point.x, point.y+field.getSize().height);
		window.setSize(300, 200);
		window.setVisible(true);
		window.toFront();
		window.setFocusableWindowState(true);
		window.requestFocus();
		list.requestFocus();
		list.setSelectedValue(field.getValue(), true);
		window.addWindowFocusListener(window);
	}

	private JPanel createContentPanel(Collection values)
	{
		JPanel panel=new JPanel(new BorderLayout());
        List valueList=Collections.EMPTY_LIST;
        if (values!=null)
        {
            valueList = new LinkedList(values);
            Collections.sort(valueList, new StringComparator());
        }
        DefaultListModel model=new DefaultListModel();
		Iterator it=valueList.iterator();
		while (it.hasNext()) model.addElement(it.next());
		SelectionListener selectionListener=new SelectionListener();
		list=new JList(model);
		list.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), "select");
		list.getActionMap().put("select", selectionListener);
		list.getInputMap(JList.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "cancel");
		list.getActionMap().put("cancel", new CancelAction());
		list.addMouseListener(selectionListener);
		panel.add(new JScrollPane(list), BorderLayout.CENTER);
		return panel;
	}

	public class CancelAction extends AbstractAction
	{
		public void actionPerformed(ActionEvent e)
		{
			field.requestFocus();
		}
	}

	public class SelectionListener extends AbstractAction implements MouseListener
	{
		public void actionPerformed(ActionEvent e)
		{
			Object value=list.getSelectedValue();
			if (value!=null)
			{
				field.setValue(value);
				field.requestFocus();
			}
		}

		public void mouseClicked(MouseEvent e)
		{
			if (e.getClickCount()>1 && e.getButton()==MouseEvent.BUTTON1)
			{
				Object value=list.getSelectedValue();
				if (value!=null)
				{
					field.setValue(value);
					field.requestFocus();
				}
				e.consume();
			}
		}

		public void mousePressed(MouseEvent e)
		{
		}

		public void mouseReleased(MouseEvent e)
		{
		}

		public void mouseEntered(MouseEvent e)
		{
		}

		public void mouseExited(MouseEvent e)
		{
		}
	}

	public class LookupWindow extends JWindow implements WindowFocusListener
	{
		public LookupWindow(Collection values)
		{
			super((Window)field.getTopLevelAncestor());
			setContentPane(createContentPanel(values));
		}

		public void windowGainedFocus(WindowEvent e)
		{
		}

		public void windowLostFocus(WindowEvent e)
		{
			dispose();
		}

		public void dispose()
		{
			field=null;
			list=null;
			super.dispose();
		}
	}
}
