/*
 * Copyright (C) 2002-2003 Stefan Stiller
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.kiwisoft.utils.gui.table;

import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Color;
import javax.swing.JTable;
import javax.swing.Icon;
import javax.swing.SwingConstants;
import javax.swing.table.DefaultTableCellRenderer;

import com.kiwisoft.utils.format.ObjectFormat;
import com.kiwisoft.utils.gui.ComponentDecorator;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.1.1.1 $, $Date: 2004/07/17 11:17:52 $
 */
public class DefaultObjectRenderer extends DefaultTableCellRenderer implements ObjectRenderer
{
    private ObjectFormat format;
    private Font font;
    private ComponentDecorator componentDecorator;

    public DefaultObjectRenderer(ObjectFormat format)
    {
        this.format = format;
    }

    public void setBackground(Color c)
    {
        super.setBackground(c);
    }

    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
    {
        RendererFactory.initializeRenderer(table, this, row);
        if (SortableTable.TOP.equals(table.getClientProperty(SortableTable.VERTICAL_CELL_ALIGNMENT)))
            setVerticalAlignment(SwingConstants.TOP);
        setHorizontalAlignment(format.getHorizontalAlignment(value));
        String text = format.format(value);
        if (text == null) text = (String) table.getClientProperty(SortableTable.NULL_STRING);
        Icon icon = format.getIcon(value);
        if (icon != null) setIcon(icon);
        else setIcon(null);
        Component rendererComponent = super.getTableCellRendererComponent(table, text, isSelected, hasFocus, row, column);
        if (font != null) rendererComponent.setFont(font);
        return rendererComponent;
    }

    protected void paintComponent(Graphics g)
    {
        super.paintComponent(g);
        if (componentDecorator != null) componentDecorator.decorateComponent(g, this);
    }

    public void setDecorator(ComponentDecorator decorator)
    {
        this.componentDecorator = decorator;
    }

    public void setTextFont(Font font)
    {
        this.font = font;
    }

    public ObjectRenderer cloneRenderer()
    {
        return new DefaultObjectRenderer(format);
    }

    public String toString()
    {
        return "DefaultObjectRenderer[format=" + format + "]";
    }
}
