/*
 * Copyright (C) 2002-2003 Stefan Stiller
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.kiwisoft.utils.gui.table;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;

import com.kiwisoft.utils.Tristate;
import com.kiwisoft.utils.gui.ComponentDecorator;
import com.kiwisoft.utils.gui.TristateBox;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.1.1.1 $, $Date: 2004/07/17 11:17:53 $
 */
public class TristateObjectRenderer extends TristateBox implements ObjectRenderer
{
	protected static Border noFocusBorder=new EmptyBorder(1, 1, 1, 1);

	private ComponentDecorator decorator;
	private Color unselectedForeground;
	private Color unselectedBackground;

	public TristateObjectRenderer()
	{
		setHorizontalAlignment(JLabel.CENTER);
	}

	public void setForeground(Color fg)
	{
		super.setForeground(fg);
		unselectedForeground=fg;
	}

	public void setBackground(Color bg)
	{
		super.setBackground(bg);
		unselectedBackground=bg;
	}

	public void setTextFont(Font font)
	{
	}

	public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
	{
        RendererFactory.initializeRenderer(table, this, row);
        if (isSelected)
		{
			super.setForeground(table.getSelectionForeground());
			super.setBackground(table.getSelectionBackground());
		}
		else
		{
			super.setForeground((unselectedForeground!=null) ? unselectedForeground : table.getForeground());
			super.setBackground((unselectedBackground!=null) ? unselectedBackground : table.getBackground());
		}
		if (value instanceof Tristate)
			setState((Tristate)value);
		else if (value instanceof Boolean)
			setState(Tristate.getTristate((Boolean)value));
		else
			setState(Tristate.UNDEFINED);
		setFont(table.getFont());

		if (hasFocus)
		{
			setBorder(UIManager.getBorder("Table.focusCellHighlightBorder"));
			if (table.isCellEditable(row, column))
			{
				super.setForeground(UIManager.getColor("Table.focusCellForeground"));
				super.setBackground(UIManager.getColor("Table.focusCellBackground"));
			}
		}
		else
		{
			setBorder(noFocusBorder);
		}

		return this;
	}

	protected void paintComponent(Graphics g)
	{
		super.paintComponent(g);
		if (decorator!=null) decorator.decorateComponent(g, this);
	}

	/**
	 * Notification from the <code>UIManager</code> that the look and feel
	 * [L&F] has changed.
	 * Replaces the current UI object with the latest version from the
	 * <code>UIManager</code>.
	 *
	 * @see javax.swing.JComponent#updateUI
	 */
	public void updateUI()
	{
		super.updateUI();
		setForeground(null);
		setBackground(null);
	}

	public void setDecorator(ComponentDecorator decorator)
	{
		this.decorator=decorator;
	}

	public ObjectRenderer cloneRenderer()
	{
		return new TristateObjectRenderer();
	}
}
